# tt_c2g.rb
#-----------------------------------------------------------------------------
# Version: 1.0.0
# Compatible: SketchUp 7 (PC)
#             (other versions untested)
#
#-----------------------------------------------------------------------------
#
# Changelog
# 1.0.0 - 13.12.2008
#       * Select Active Instances
#       * Select Instances
#       * Select Active Group Copies
#       * Select Group Copies
#       * Convert Group Copies to Instances
#
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------
# Add some menu items to access this
if not file_loaded?('tt_c2g.rb')
	# --- Commands --- #
	# Component
	cmd_make_groups 		= UI::Command.new('Make Groups')		{ TT_Component_to_Group.make_groups() }
	cmd_make_nested_groups	= UI::Command.new('Make Nested Groups')	{ TT_Component_to_Group.make_nested_groups() }
		
	UI.add_context_menu_handler do |menu|
		if TT_Component_to_Group.selected_component?
			menu.add_item(cmd_make_groups)
			menu.add_item(cmd_make_nested_groups)
		end
	end
end

#-----------------------------------------------------------------------------
file_loaded('tt_c2g.rb')
#-----------------------------------------------------------------------------

module TT_Component_to_Group
	
	# Converts all components in selection to groups
	def self.make_groups
		model = Sketchup.active_model
		sel = model.selection
		space = model.active_entities 
		
		Helping_Hand.start_operation('Make Groups')
		
		sel.each { |e|
			next if !e.kind_of?(Sketchup::ComponentInstance)
			self.component_to_group(e, space)
		}
		
		model.commit_operation
	end
	
	# Convert all components and nested components to groups
	# (!) Crash upon closing the model
	def self.make_nested_groups()
		model = Sketchup.active_model
		sel = model.selection
		space = model.active_entities
		
		Helping_Hand.start_operation('Make Nested Groups')
		
		components = self.c2g(sel, space)
		#puts components.inspect
		#components.each { |e|
		#	e.make_unique
		#}
		#components.each { |e|
		#	self.component_to_group(e)
		#}
		
		model.commit_operation
	end
	def self.c2g(entities, space)
		#puts '### C2G ###'
		#i = []
		entities.each { |e|
			next if !e.kind_of?(Sketchup::ComponentInstance)
			#i.push(e)
			#puts 'Converting Component...'
			g = self.component_to_group(e, space)
			self.c2g(g.entities, g.entities)
			#i = i + self.c2g(e.entities)
		}
		#return i
	end
	
	# Converts the given Instance to a group
	def self.component_to_group(instance, space)
		puts 'instance: ' + instance.definition.name
		instance.make_unique
		#parent = instance.parent
		#puts 'parent: ' + parent.inspect
		puts 'entities: ' + space.inspect
		puts 'parent: ' + space.parent.inspect
		
		group = space.add_group(instance)
		instance.explode
		return group
	end
	
	### VALIDATIONS ### ------------------------------------------------------

	def self.selected_component?
		sel = Sketchup.active_model.selection
		
		sel.each { |e|
			return true if e.kind_of?(Sketchup::ComponentInstance)
		}
		return false
	end
	
	### HELPER METHODS ### ---------------------------------------------------
	
	module Helping_Hand
		def self.start_operation(name)
			model = Sketchup.active_model
			# Make use of the SU7 speed boost with start_operation while
			# making sure it works in SU6.
			if Sketchup.version.split(".")[0].to_i >= 7
				model.start_operation(name, true)
			else
				model.start_operation(name)
			end
		end
	end # module Helping_Hand
end # module Select_Toys